@@ -36,6 +36,13 @@ LOG = logging.getLogger(__name__)
 
 VIRTFN_RE = re.compile("virtfn(\d+)")
 
+# phys_port_name only contains the VF number
+INT_RE = re.compile("^(\d+)$")
+# phys_port_name contains VF## or vf##
+VF_RE = re.compile("vf(\d+)", re.IGNORECASE)
+# phys_port_name contains PF## or pf##
+PF_RE = re.compile("pf(\d+)", re.IGNORECASE)
+
 
 def _ovs_vsctl(args, timeout=None):
     full_args = ['ovs-vsctl']
@@ -53,8 +60,7 @@ def _ovs_vsctl(args, timeout=None):
 def _create_ovs_vif_cmd(bridge, dev, iface_id, mac,
                         instance_id, interface_type=None,
                         vhost_server_path=None):
-    cmd = ['--', '--if-exists', 'del-port', dev, '--',
-            'add-port', bridge, dev,
+    cmd = ['--', '--may-exist', 'add-port', bridge, dev,
             '--', 'set', 'Interface', dev,
             'external-ids:iface-id=%s' % iface_id,
             'external-ids:iface-status=active',
@@ -88,10 +94,11 @@ def update_ovs_vif_port(dev, mtu=None, interface_type=None, timeout=None):
 
 
 @privsep.vif_plug.entrypoint
-def delete_ovs_vif_port(bridge, dev, timeout=None):
+def delete_ovs_vif_port(bridge, dev, timeout=None, delete_netdev=True):
     _ovs_vsctl(['--', '--if-exists', 'del-port', bridge, dev],
                timeout=timeout)
-    _delete_net_dev(dev)
+    if delete_netdev:
+        _delete_net_dev(dev)
 
 
 def device_exists(device):
@@ -99,6 +106,12 @@ def device_exists(device):
     return os.path.exists('/sys/class/net/%s' % device)
 
 
+def interface_in_bridge(bridge, device):
+    """Check if an ethernet device belongs to a Linux Bridge."""
+    return os.path.exists('/sys/class/net/%(bridge)s/brif/%(device)s' %
+                          {'bridge': bridge, 'device': device})
+
+
 def _delete_net_dev(dev):
     """Delete a network device only if it exists."""
     if device_exists(dev):
@@ -145,6 +158,7 @@ def ensure_bridge(bridge):
         processutils.execute('brctl', 'addbr', bridge)
         processutils.execute('brctl', 'setfd', bridge, 0)
         processutils.execute('brctl', 'stp', bridge, 'off')
+        processutils.execute('brctl', 'setageing', bridge, 0)
         syspath = '/sys/class/net/%s/bridge/multicast_snooping'
         syspath = syspath % bridge
         processutils.execute('tee', syspath, process_input='0',
@@ -163,7 +177,8 @@ def ensure_bridge(bridge):
 @privsep.vif_plug.entrypoint
 def delete_bridge(bridge, dev):
     if device_exists(bridge):
-        processutils.execute('brctl', 'delif', bridge, dev)
+        if interface_in_bridge(bridge, dev):
+            processutils.execute('brctl', 'delif', bridge, dev)
         processutils.execute('ip', 'link', 'set', bridge, 'down')
         processutils.execute('brctl', 'delbr', bridge)
 
@@ -224,6 +239,35 @@ def _ovs_supports_mtu_requests(timeout=None):
     return True
 
 
+def _parse_vf_number(phys_port_name):
+    """Parses phys_port_name and returns VF number or None.
+
+    To determine the VF number of a representor, parse phys_port_name
+    in the following sequence and return the first valid match. If none
+    match, then the representor is not for a VF.
+    """
+    match = INT_RE.search(phys_port_name)
+    if match:
+        return match.group(1)
+    match = VF_RE.search(phys_port_name)
+    if match:
+        return match.group(1)
+    return None
+
+
+def _parse_pf_number(phys_port_name):
+    """Parses phys_port_name and returns PF number or None.
+
+    To determine the PF number of a representor, parse phys_port_name in
+    the following sequence and return the first valid match. If none
+    match, then the representor is not for a PF.
+    """
+    match = PF_RE.search(phys_port_name)
+    if match:
+        return match.group(1)
+    return None
+
+
 def get_representor_port(pf_ifname, vf_num):
     """Get the representor netdevice which is corresponding to the VF.
 
@@ -269,11 +313,17 @@ def get_representor_port(pf_ifname, vf_num):
             continue
 
         try:
-            with open(device_sw_id_file, 'r') as fd:
-                representor_num = fd.readline().rstrip()
+            with open(device_port_name_file, 'r') as fd:
+                phys_port_name = fd.readline().rstrip()
         except (OSError, IOError):
             continue
 
+        representor_num = _parse_vf_number(phys_port_name)
+        # Note: representor_num can be 0, referring to VF0
+        if representor_num is None:
+            continue
+
+        # At this point we're confident we have a representor.
         try:
             if int(representor_num) == int(vf_num):
                 return device
@@ -293,18 +343,41 @@ def _get_sysfs_netdev_path(pci_addr, pf_interface):
     return "/sys/bus/pci/devices/%s/net" % (pci_addr)
 
 
-def get_ifname_by_pci_address(pci_addr, pf_interface=False):
+def _is_switchdev(netdev):
+    """Returns True if a netdev has a readable phys_switch_id"""
+    try:
+        sw_id_file = "/sys/class/net/%s/phys_switch_id" % netdev
+        with open(sw_id_file, 'r') as fd:
+            phys_switch_id = fd.readline().rstrip()
+        if phys_switch_id != "" and phys_switch_id is not None:
+            return True
+    except (OSError, IOError):
+        return False
+    return False
+
+
+def get_ifname_by_pci_address(pci_addr, pf_interface=False, switchdev=False):
     """Get the interface name based on a VF's pci address
 
+    :param pci_addr: the PCI address of the VF
+    :param pf_interface: if True, look for the netdev of the parent PF
+    :param switchdev: if True, ensure that phys_switch_id is valid
+
+    :returns: netdev interface name
+
     The returned interface name is either the parent PF or that of the VF
     itself based on the argument of pf_interface.
     """
     dev_path = _get_sysfs_netdev_path(pci_addr, pf_interface)
+    # make the if statement later more readable
+    ignore_switchdev = not switchdev
     try:
-        dev_info = os.listdir(dev_path)
-        return dev_info.pop()
+        for netdev in os.listdir(dev_path):
+            if ignore_switchdev or _is_switchdev(netdev):
+                return netdev
     except Exception:
         raise exception.PciDeviceNotFoundById(id=pci_addr)
+    raise exception.PciDeviceNotFoundById(id=pci_addr)
 
 
 def get_vf_num_by_pci_address(pci_addr):
